use actix_web::{web, HttpResponse, Responder};
use serde::Deserialize;
use sqlx::PgPool;

use crate::models::{Assignment, Event, Function};

#[derive(Deserialize)]
pub struct NewAssignmentsForm {
    event: i32,
    wachhabender: Option<i32>,
    posten: Vec<i32>
}

#[actix_web::post("/assignments/new")]
pub async fn post(pool: web::Data<PgPool>, form: web::Form<Vec<(String, String)>>) -> impl Responder {
    let event_id = form.iter().find(|x| x.0 == "event").unwrap().1.parse().unwrap();
    let wachhabender = form.iter().find(|x| x.0 == "wachhabender");
    let posten: Vec<i32> = form.iter().filter(|x| x.0 == "posten").map(|x| x.1.parse().unwrap()).collect();

    let event = Event::read_by_id_including_location(&pool, event_id).await.unwrap(); // TODO: Check if location is needed

    if event.voluntary_wachhabender && wachhabender.is_some() && posten.contains(&wachhabender.unwrap().1.parse().unwrap()) {
        return HttpResponse::BadRequest().body("Wachhabender kann nicht zugleich Posten sein!");
    }

    let mut joined_ids = posten.clone();
    if let Some((_,id)) = wachhabender {
        joined_ids.push(id.parse().unwrap());
    }

    for availabillity_id in joined_ids {
        let assignments = Assignment::read_by_availabillity(pool.get_ref(), availabillity_id).await.unwrap();

        let mut can_be_used = true;


        for assignment in assignments {
            if event.start_time >= assignment.start_time && event.start_time <= assignment.end_time {
            } else {
                can_be_used = false;
                break;
            }
        }

        if !can_be_used {
            return HttpResponse::BadRequest().body("availabillity time slot bereits genutzt!");
        }
    }

    if let Some((_,id)) = wachhabender {
        Assignment::create(pool.get_ref(), event.id, id.parse().unwrap(), Function::Wachhabender, event.start_time, event.end_time).await.unwrap();
    }

    for id in posten {
        Assignment::create(pool.get_ref(), event.id, id, Function::Posten, event.start_time, event.end_time).await.unwrap();
    }

    HttpResponse::Ok().finish()
}
