use actix_web::{web, HttpResponse, Responder};
use askama::Template;
use askama_actix::TemplateToResponse;
use sqlx::PgPool;

use crate::models::{Area, Role, User};

#[derive(Template)]
#[template(path = "location/new.html")]
pub struct NewLocationTemplate {
    user: User,
    areas: Option<Vec<Area>>,
}

#[actix_web::get("/locations/new")]
pub async fn get(user: web::ReqData<User>, pool: web::Data<PgPool>) -> impl Responder {
    if user.role == Role::AreaManager || user.role == Role::Admin {
        let mut areas = None;

        if user.role == Role::Admin {
            areas = Some(Area::read_all(pool.get_ref()).await.unwrap());
        }

        let template = NewLocationTemplate {
            user: user.into_inner(),
            areas,
        };

        return template.to_response();
    }

    return HttpResponse::Unauthorized().finish();
}
