use actix_identity::Identity;
use actix_web::{web, HttpResponse, Responder};
use sqlx::PgPool;

use crate::{endpoints::IdPath, models::{Role, User}};

#[actix_web::delete("/users/delete/{id}")]
pub async fn delete(user: Identity, pool: web::Data<PgPool>, path: web::Path<IdPath>) -> impl Responder {
    let current_user = User::read_by_id(pool.get_ref(), user.id().unwrap().parse().unwrap())
        .await
        .unwrap();

    if current_user.role != Role::AreaManager && current_user.role != Role::Admin {
        return HttpResponse::Unauthorized().finish();
    }

    if let Ok(user_in_db) = User::read_by_id(pool.get_ref(), path.id).await {
        if current_user.role == Role::AreaManager && current_user.area_id != user_in_db.area_id {
            return HttpResponse::Unauthorized().finish();
        }

        if user_in_db.locked {
            if let Ok(_) = User::delete(pool.get_ref(), user_in_db.id).await {
                return HttpResponse::NoContent().finish();
            }
        }
    }

    HttpResponse::BadRequest().finish()
}
