use actix_web::{web, HttpResponse, Responder};
use serde::Deserialize;
use serde_json::value::Value;
use sqlx::PgPool;

use crate::{
    endpoints::IdPath,
    models::{Role, User},
};

#[derive(Deserialize)]
pub struct JsonPatchDoc {
    op: String,
    path: String,
    value: Value,
}

#[actix_web::patch("/users/edit/{id}")]
pub async fn patch(
    user: web::ReqData<User>,
    pool: web::Data<PgPool>,
    path: web::Path<IdPath>,
    patch_docs: web::Json<Vec<JsonPatchDoc>>,
) -> impl Responder {

    let is_superuser = user.role != Role::AreaManager && user.role != Role::Admin;

    if let Ok(user_in_db) = User::read_by_id(pool.get_ref(), path.id).await {
        if user.role == Role::AreaManager && user.area_id != user_in_db.area_id {
            return HttpResponse::Unauthorized().finish();
        }

        let mut changed = false;

        let mut locked: Option<bool> = None;
        let mut receive_notifications: Option<bool> = None;

        for doc in patch_docs.iter() {
            if doc.op.as_str() != "replace" {
                continue;
            }

            match doc.path.as_str() {
                "/locked" => {
                    if !is_superuser {
                        return HttpResponse::Unauthorized().finish();
                    }
                    changed = true;
                    if let Value::Bool(b) = doc.value {
                        locked = Some(b);
                    }
                }
                "/receiveNotifications" => {
                    changed = true;
                    if let Value::Bool(b) = doc.value {
                        receive_notifications = Some(b)
                    }
                }
                _ => return HttpResponse::BadRequest().body("Other PATCH paths are not supported!")
            };
        }

        if changed {
            if let Ok(_) = User::update(
                pool.get_ref(),
                path.id,
                None,
                None,
                None,
                None,
                None,
                None,
                None,
                receive_notifications,
                locked,
            )
            .await
            {
                return HttpResponse::Ok().body("");
            }
        } else {
            return HttpResponse::Ok().body("");
        }
    }

    HttpResponse::BadRequest().body("Fehler bei User PATCH")
}
