use actix_identity::Identity;
use actix_web::{http::header::LOCATION, web, HttpResponse, Responder};
use serde::Deserialize;
use sqlx::PgPool;

use crate::{auth::utils::generate_salt_and_hash_plain_password, models::{Function, Role, User}};

#[derive(Deserialize)]
pub struct NewUserForm {
    email: String,
    name: String,
    password: String,
    role: u8,
    function: u8, 
    area: Option<i32>
}

#[actix_web::post("/users/new")]
pub async fn post_new(user: Identity, pool: web::Data<PgPool>, form: web::Form<NewUserForm>) -> impl Responder {
    let current_user = User::read_by_id(pool.get_ref(), user.id().unwrap().parse().unwrap()).await.unwrap();

    if current_user.role != Role::AreaManager && current_user.role != Role::Admin {
        return HttpResponse::Unauthorized().finish();
    }

    let mut area_id = current_user.area_id;

    if current_user.role == Role::Admin {
        if let Some(id) = form.area {
            area_id = id;
        }
    }

    if let Ok((hash, salt)) = generate_salt_and_hash_plain_password(&form.password) {
        if let Ok(role) = Role::try_from(form.role) {
            if let Ok(function) = Function::try_from(form.function) {
                match User::create(pool.get_ref(), &form.name, &form.email, &hash, &salt, role, function, area_id).await {
                    Ok(_) => return HttpResponse::Found().insert_header((LOCATION, "/users")).finish(),
                    Err(err) => println!("{}", err)
                }
            }
        }
    }

    return HttpResponse::BadRequest().body("Fehler beim Erstellen des Nutzers");
}
